// Audio/Video Test for 32X - by  Chilly Willy

#include <sys/types.h>
#include <string.h>
#include <math.h>

#include "32x.h"
#include "debug_32x.h"

#define NUM_SAMPS 882
#define MAX_NUM_SAMPLES 882
#define SAMPLE_RATE     44100
#define SAMPLE_CENTER   258

unsigned short __attribute__((aligned(16))) sndbuf[MAX_NUM_SAMPLES*2]; // one buffer of NUM_SAMPS words of stereo pwm audio

extern void cache_control(int mode);

int main( void )
{
    volatile unsigned short *frameBuffer16 = &MARS_FRAMEBUFFER;
    int i, j, s;

    Debug32xInit(MARS_VDP_MODE_32K);

	// generate sine wave in buffer
	for (i = 0; i < NUM_SAMPS; i++)
	{
		sndbuf[i*2] = (unsigned short)(sin((double)i * M_PI / 22.05) * 256.0 + (double)SAMPLE_CENTER);
		sndbuf[i*2+1] = (unsigned short)(sin((double)i * M_PI / 22.05) * 256.0 + (double)SAMPLE_CENTER);
	}

	// draw greyscale/color ramps
	for (s = 0; s < 32; s++)
	{
		for (j = 0; j < 16; j++)
		{
			for (i = 0; i < 10; i++)
			{
				frameBuffer16[(j + 16) * 320 + i + s * 10 + 256] = COLOR(s, s, s);
				frameBuffer16[(j + 48) * 320 + i + s * 10 + 256] = COLOR(s, 0, 0);
				frameBuffer16[(j + 80) * 320 + i + s * 10 + 256] = COLOR(0, s, 0);
				frameBuffer16[(j + 112) * 320 + i + s * 10 + 256] = COLOR(0, 0, s);
			}
		}
	}
	Debug32xScreenFlip(1);

    MARS_SYS_COMM4 = 0xFFFF; // init audio DMA
    while (MARS_SYS_COMM4 != 0) ; // wait for audio start

    while (1)
    {
    }

    return 0;
}


void slave(void)
{
    while (MARS_SYS_COMM4 != 0xFFFF) ; // wait for audio init command

    while (1)
    {
        // only do sound when sound subsytem initialized
        while (MARS_SYS_COMM4 != 0xFFFF)
        {
            // start DMA on buffer
            SH2_DMA_SAR1 = (unsigned long)sndbuf | 0x20000000;
            SH2_DMA_TCR1 = NUM_SAMPS; // number longs
            SH2_DMA_CHCR1 = 0x18E1; // dest fixed, src incr, size long, ext req, dack mem to dev, dack hi, dack edge, dreq rising edge, cycle-steal, dual addr, intr disabled, clear TE, dma enabled

            // wait on DMA
            while (!(SH2_DMA_CHCR1 & 2)) ; // wait on TE

        }
        if (MARS_SYS_COMM4 == 0xFFFF)
        {
            unsigned short sample, ix;

            // init DMA
            SH2_DMA_SAR0 = 0;
            SH2_DMA_DAR0 = 0;
            SH2_DMA_TCR0 = 0;
            SH2_DMA_CHCR0 = 0;
            SH2_DMA_DRCR0 = 0;
            SH2_DMA_SAR1 = 0;
            SH2_DMA_DAR1 = 0x20004034; // storing a long here will set left and right
            SH2_DMA_TCR1 = 0;
            SH2_DMA_CHCR1 = 0;
            SH2_DMA_DRCR1 = 0;
            SH2_DMA_DMAOR = 1; // enable DMA

            // init the sound hardware
            MARS_PWM_MONO = 1;
            MARS_PWM_MONO = 1;
            MARS_PWM_MONO = 1;
            if (MARS_VDP_DISPMODE & MARS_NTSC_FORMAT)
                MARS_PWM_CYCLE = (((23011361 << 1)/SAMPLE_RATE + 1) >> 1) + 1; // for NTSC clock
            else
                MARS_PWM_CYCLE = (((22801467 << 1)/SAMPLE_RATE + 1) >> 1) + 1; // for PAL clock
            MARS_PWM_CTRL = 0x0185; // TM = 1, RTP, RMD = right, LMD = left

            sample = 1;
            /* ramp up to SAMPLE_CENTER to avoid click in audio (real 32X) */
            while (sample < SAMPLE_CENTER)
            {
                for (ix=0; ix<SAMPLE_RATE/SAMPLE_CENTER; ix++)
                {
                    while (MARS_PWM_MONO & 0x8000) ; // wait while full
                    MARS_PWM_MONO = sample;
                }
                sample++;
            }
            MARS_SYS_COMM4 = 0; // audio initialized and ready
        }
    }
}
